
/*
	HE_Dos.h

	This is C code

	This stuff is colored with HEDsk_,
	defines with HE, no namespace in C

*/

/*
		--------FAT INFO----------
Boot Sector
	Start of Partition

Fat Tables
	Start + # of Reserved Sectors

Root Directory Entry
	Start + # of Reserved + (# of Sectors Per FAT * 2)

Data Area (Starts with Cluster #2)
	Start + # of Reserved + (# of Sectors Per FAT * 2)
		+ ((Maximum Root Directory Entries * 32) / Bytes per Sector)

http://home.teleport.com/~brainy/fat16.htm

~~~~~
Offset  Length  	  Value
0		8 bytes 	  Name
8 		3 bytes 	  Extension
11	 	byte 	  Attribute (00ARSHDV)
22		word 	  Time
24		word 	  Date
26		word 	  Cluster (desc. below)
28		dword 	  File Size

~~~
directory attributes

0	0x01	Read Only
1	0x02	Hidden
2	0x04	System
3	0x08	Volume Label (only allowed in entry in root directory)
4	0x10	Subdirectory
5	0x20	Archive
6	0x40	Device (internal use only, never found on disk)
7	0x80	Unused
DOS file name (padded with spaces)

The first byte can have the following special values:

0x00	Entry is available and no subsequent entry is in use
0x05	Initial character is actually 0xE5
0x2E	'Dot' entry; either '.' or '..'
0xE5	Entry has been previously erased and is available.
		File undelete utilities must replace this character with a
		regular character as part of the undeletion process.

*/

//Offsets MBR
#define MBR_LBA0		0x01c6

//Offsets vol info
#define HEFAT_RESERVED			14 //0x0e uint16_t
#define HEFAT_SECPERFAT			22 //0x16 uint16_t
#define HEFAT_DIRENTRIES		26 //0x1a uint16_t //?Number of heads?
#define HEFAT_SECPERCLST		13 //0x0d uint8_t

//Sizes
#define HEFAT_SZF_FNAME			11
#define HEFAT_SZF_DIRENT		32
#define HEFAT_ENTRIESPERSEC		16

//internal specific
#define HEDS_SECTORSIZE			512
#define HEDS_HALFSECMASK		0xff //255
#define HEDS_SECTOR_SHIFT		9
#define HEDS_SECTORMASK			0x1ff //512
#define HEDS_POINTCLSTMSK		0x7f00
#define HEDS_FIRSTFATPOS		0x0003
#define HEDS_DIRCNTMASK			0x0f
#define HEDS_ENDCLUSTMSK		0xffff
#define HEDS_ENDFATMASK			0x7f
#define HEDS_FATSECSHIFT		8

//Offsets file info
#define HEDS_FILE_DIR_CLUST		26 //0x1a
#define HEDS_FILE_SIZE			28 //0x1c

//volume flags
//sector needs to be written
#define HEDS_VF_DIRTY		0x01

typedef struct _tagHEDskVolInfo
{
	uint32_t	bootSector;
	uint32_t	rootFat;
	uint32_t	rootDir;
	uint32_t	rootData;
	//Used by GetSector and SetSector for caching
	uint32_t	curSector;
	//more economical to calc this once
	uint32_t	bytesPerClust;
	//and 11/1/9
	//uint16_t	secPerClustShift;
	//limit of root dir search
	uint16_t	dirEntries;
	//This is about writting a file so keep a free fat position
	uint16_t	freeFatPos;
	//Sectors per Cluster
	uint8_t		secPerClust;
	//Flags, of course
	uint8_t		volFlags;

} HEDskVolInfo, *pHEDskVolInfo;


typedef struct _tagHEDskFileInfo
{
	//current size of file
	uint32_t	size;
	//Current file pointer by cluster sectorPos and pointer
	uint32_t	curSectorPos;
	//Always less than 512, pointer into curSector
	uint16_t	curPointer;
	//so we don't have to calc it
	uint16_t	curClust;
	//So it doesn't have to be looked up more than once
	//By uint16_t, assumes partition not too far out
	uint16_t	dirSect;
	uint16_t	dirPos;

} HEDskFileInfo, *pHEDskFileInfo;

//Initialize the drive information
bool HEDsk_LoadVolumeInfo( );

//Hunts for next avaliable free cluster and sets location in Volume info
void HEDsk_GetFreeFat( );

//add a new cluster to file
void HEDsk_AddCluster( pHEDskFileInfo pInfo );

/*
	HEDsk_GetFileInfo(...)
	[in] Using cananonical name as "TEST    TXT" does not need trailing null
	[out] Populates diskFileInfo if file found
	Same as: OpenFile(...)
	does not search sub directories, file needs to exsist and be in root
*/
bool HEDsk_GetFileInfo( pHEDskFileInfo pInfo, uint8_t* psCanonicalName );

//This only sets the DskFileInfo and does not read the data sector
//Warning: No check here that pos is <= file size for now
void HEDsk_SeekFile( pHEDskFileInfo pInfo, uint32_t pos );

//Read from the current file position, count to buf
bool HEDsk_ReadFile( pHEDskFileInfo pInfo, uint8_t* buf, uint16_t count );

//Writes at the current file position and will alloc as file grows
bool HEDsk_WriteFile( pHEDskFileInfo pInfo, uint8_t* buf, uint16_t count );

//Like 'File.Close' but info is still really open, just a flush
//Updates file size in dir. TODO Set the modified date ?
bool HEDsk_FlushFile( pHEDskFileInfo pInfo );

//there must be one
extern HEDskVolInfo heVol;

//helpers...
bool HEDsk_strcmp( const uint8_t* a, const uint8_t* b, uint16_t count );
void HEDsk_memcpy( uint8_t* pDest, uint8_t* pSource, uint16_t count );

//or use stdlib...
//#define HEDsk_strcmp( a, b, c ) ( ! str?cmp( a, b, c ) )
//#define HEDsk_memcpy memcpy


