#include  "Device_430.h"
#include "spi.h"
#include "sd.h"

/* SPI SD initialization sequence:
* CMD0
* CMD55
* ACMD41
* CMD58
*/

bool sd_initialize( )
{
	// Delay for at least 74 clock cycles. This means to actually
	// *clock* out at least 74 clock cycles with no data present on
	// the clock. In SPI mode, send at least 10 idle bytes (0xFF). 

	//initialize the SPI port
	spi_initialize( );
	
	//Start out with a slow SPI clock, 400kHz, as required by the SD spec
	spi_set_divisor( SD_CLK_SLOW );

	//spi_cs_assert( );
	SPI_CS_ASSERT;
	sd_delay( 100 );
	
	SPI_CS_DEASSERT;
	sd_delay( 10 );
	
	// Put the card in the idle state
	if( ! sd_send_command( SDCMD_0, 0 ) )
		return false;

	//wait until idle clear
	int j;
	for( j= 0; j < SD_IDLE_WAIT_MAX; ++j )
	{
		if( sd_send_command( SDCMD_55, 0 ) )
		{
			/* Tell the card to send its OCR */
			sd_send_command( SDCMD_A41, 0 );
		}
		
		if( ! response[0] )
			break;
	}
	if( j == SD_IDLE_WAIT_MAX )
		return false;

	if( ! sd_send_command( SDCMD_58, 0 ) )
		return false;

	//Set the maximum SPI clock rate possible
	spi_set_divisor( 2 );

	return true;
}

bool sd_read_block( DWORD blockaddr, const BYTE* data )
{
	unsigned long int i= 0;
	unsigned char tmp;
	block_count= 0;
	
	// Adjust the block address to a linear address
	blockaddr<<= SD_BLOCKSIZE_NBITS;

	//Send block read command
	if( ! sd_send_command( SDCMD_17, blockaddr ) )
		return false;
		
	// Check for an error, like a misaligned read
	if( response[0] )
		return false;
		
	// Re-assert CS to continue the transfer
	SPI_CS_ASSERT;

	// Wait for the token
	for( i= 0, tmp= 0xff; ( tmp == 0xFF ) && i < SD_TIMEOUT_READ; ++i )
		tmp= spi_rcv_byte( );

	if( ! ( tmp & MSK_TOK_DATAERROR ) )
	{
		// The card returned an error response. Bail and return false
		// Clock out a byte before returning
		spi_send_byte( 0xFF );

		return 0;
	}
	
	//No dma, just do bulk transfer in code
	for( block_count= 0; block_count < SD_BLOCKSIZE; ++block_count )
		secBuf[ block_count ]= spi_rcv_byte( );

	return 1;
}

bool sd_write_block( DWORD blockaddr, const BYTE *data )
{
	// Adjust the block address to a linear address
	blockaddr <<= SD_BLOCKSIZE_NBITS;

	// Pack the address
	//sd_packarg( argument, blockaddr );
	
	if( ! sd_send_command( SDCMD_24, blockaddr ) )
		return false;
		
	// Check for an error, like a misaligned write
	if( response[0] )
		return false;

	// Re-assert CS to continue the transfer
	SPI_CS_ASSERT;
	
	// The write command needs an additional 8 clock cycles before the block write is started
	spi_rcv_byte( );
	
	// Kick off the transfer by sending the first byte, the "start block" token
	SD_UTXBUF = SD_TOK_WRITE_STARTBLOCK;

	//Write the block
	for( block_count= 0; block_count < SD_BLOCKSIZE; ++block_count )
		spi_send_byte( secBuf[ block_count ] );

	//need to pad some clocks or get Erase_Seq_Error
	//	on the next write, and a rcv_byte...
	sd_delay( 4 );
	
	//TODO ? might get hung here? Or should this be left for next op?
	for( ; spi_rcv_byte( ) != 0xFF; )
		;

	return true;
}

bool sd_send_command( WORD cmd, DWORD arg )
{
	volatile int i;
	int rpos;
	unsigned char tmp= 0;

	SPI_CS_ASSERT;
	spi_send_byte( cmd >> 8 );

	for( i= 3; i >= 0 ; --i )
		spi_send_byte( ( (BYTE*)&arg )[ i ] );

	// This is the CRC. It only matters what we put here for the first
	//command. Otherwise, the CRC is ignored for SPI mode unless we
	//enable CRC checking.
	spi_send_byte( 0x95 );
	
	rpos= (BYTE)cmd - 1;
	
	/* Wait for a response. A response can be recognized by the
	start bit (a zero) */
	for( i= 0; i < SD_CMD_TIMEOUT; ++i )
	{
		tmp= spi_rcv_byte( );
		if( !( tmp & 0x80 ) )
			break;
	}
	/* Just bail if we never got a response */
	if( i >= SD_CMD_TIMEOUT )
	{
		SPI_CS_DEASSERT;
		return false;
	}
	for(; rpos >= 0; --rpos )
	{
		response[ rpos ]= tmp;
		tmp= spi_rcv_byte( );
	}

	SPI_CS_DEASSERT;
	return true;
}

void sd_delay( BYTE number )
{
	//clocks out idle bytes...
	for( ; number--; )
		spi_send_byte( 0xFF );
}

